/*
 * An opener for a "host" file provided via the C library.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <map>
#include <mutex>
#include <string>

#include <fsserver/opener_resource.h>



/* Collection data types. */

typedef std::map<std::string, fileid_t> HostFileIdentifiers;



/* Support for providing access to files. */

class HostFileOpener : public OpenerResource
{
protected:
    std::mutex _lock;

    /* File identifier register. */

    HostFileIdentifiers _fileids;

    virtual fileid_t _get_fileid(const char *path, bool create);

    /* Convenience methods determining different object types. */

    virtual bool accessing_directory(const char *path, flags_t flags, fileid_t fileid);

    virtual bool accessing_file(const char *path, flags_t flags, fileid_t fileid);

public:
    explicit HostFileOpener(FilePaging *paging)
    : OpenerResource(paging)
    {
    }

    virtual ~HostFileOpener();

    /* File opening methods. */

    virtual long get_fileid(const char *path, flags_t flags, fileid_t *fileid);

    virtual long make_accessor(const char *path, flags_t flags, fileid_t fileid,
                               Accessor **accessor);

    virtual long make_directory_accessor(const char *path, flags_t flags,
                                         fileid_t fileid,
                                         DirectoryAccessor **accessor);
};

// vim: tabstop=4 expandtab shiftwidth=4
